<?php
/**
 * Supr3.me Crypter Class
 * Used to properly create encrypted passwords which can be deciphered
 *
 * @author Supr3.me <tech@supr3.me>
 */

/**
 * Static Encryption Class
 *
 * Contains encryption methods that can be accessed statically
 * without creating an instance of this class
 */
class Crypter
{

    /**
     * The encryption cypher to use
     *
     * @var string
     */
    public static $sCipher = 'AES-128-CBC';

    /**
     * The block size of the cypher
     *
     * @var int
     */
    public static $iBlock = 32;

    /**
     * Encryption key to use
     *
     * @var string Encryption key
     */
    public static $sKey = null;

    /**
     * Default encryption key to use
     *
     * @var string Default encryption key, will normally be overridden
     */
    private static $sDefaultKey = ']#_W(1BDBgxW:^+K?L9\k.L]\%TTfEzWeUX.L5`5!*|{P#e6!f$V%200}O([ESr;c6qnaH#-Dum;indo';

    /**
     * Encrypt a string using mcrypt
     *
     * The string will be encrypted using the AES-256 scheme and will be base64 encoded.
     *
     * @param string $mInput Unencrypted value to encrypt
     *
     * @return string The encrypted string
     * @throws Exception
     */
    public static function encrypt($mInput)
    {

        # serialize the input
        $sInput = serialize($mInput);

        # create initialisation vector
        $iIV = random_bytes(self::_iv_size());

        // encrypt our data
        $sInput = openssl_encrypt($sInput, self::$sCipher, self::keyGet(), 0, $iIV);

        # return encrypted value
        return base64_encode($iIV . $sInput);
    }

    /**
     * Decrypt a string using mcrypt
     *
     * @param string $mInput
     *
     * @return bool|mixed
     */
    public static function decrypt(string $sInput)
    {

        # check if the input is null
        if (empty($sInput)) {
            return false;
        }

        // decode the encoded value
        $sInput = base64_decode($sInput);

        // separate out the input vector and encrypted string
        $iIV = substr($sInput, 0, self::_iv_size());
        $sInput = substr($sInput, self::_iv_size());

        // decrypt our data
        $sInput = openssl_decrypt($sInput, self::$sCipher, self::keyGet(), 0, $iIV);

        $return = unserialize($sInput);

        // return our data
        if (!$return) {
            return unserialize($sInput);
        } else {
            return $return;
        }
    }

    /**
     * Gets system key based on a number of factors
     *
     * @return string System security key
     */
    public static function keyGet()
    {
        if (strlen(self::$sKey)) {
            # key that has been specified
            $sKey = self::$sKey;
        } elseif (defined('AUTH_SALT')) {
            # the default system user password hash secret
            $sKey = AUTH_SALT;
        } else {
            # default Crypter key
            $sKey = self::$sDefaultKey;
        }

        # return a 24 char md5 of the key
        return md5($sKey);
    }

    /**
     * Return our IV size
     *
     * @return int
     */
    private static function _iv_size()
    {
        return openssl_cipher_iv_length(self::$sCipher);
    }
}