<?php
/**
 * Supr3me activity log table
 *
 * @author Supr3.me <tech@supr3.me>
 */

class Supr3meLogTable extends WP_List_Table {

    public
        $parentClass = null;


    /**
     * Override the parent constructor to pass our own arguments
     */
    public function __construct ($parentClass) {

        // set the parent class
        $this->parentClass = $parentClass;

        // return the parent constructor with a couple of extras
        parent::__construct(array(
            'singular'      => 'Login Record',
            'plural'        => 'Login Records',
            'ajax'          => false
        ));
    }

    /**
     * Add extra items to the table navigation rows (top and bottom)
     * @param  string $which top or bottom
     * @return void
     */
    public function extra_tablenav($which) {
        // create a purge all input field
        echo '<div class="alignleft actions">';
        echo '<button type="submit" name="action" value="deleteall" class="button action" style="margin-top: 1px">Purge All</button>';
        echo '</div>';
    }

    /**
     * Create the 'cb' column data
     * @param  object $item The current row data
     * @return string
     */
    public function column_cb($item){
        return sprintf(
            '<input type="checkbox" name="%s[]" value="%s" />',
            $this->_args['singular'],
            $item->id
        );
    }

    /**
     * Create the 'status' column data
     * @param  object $item The current row data
     * @return string
     */
    public function column_status($item) {
        return sprintf(
            '<span class="dashicons dashicons-%s" style="color: %s"></span>',
            $item->loginsuccess ? 'yes' : 'no-alt',
            $item->loginsuccess ? 'darkgreen' : 'darkred'
        );
    }

    /**
     * Create the 'timestamp' column data
     * @param  object $item The current row data
     * @return string
     */
    public function column_timestamp($item) {
        // return formatted time
        return sprintf(
            '%s<br /><small>%s ago</small>',
            $item->createdts,
            human_time_diff(mysql2date('U', $item->createdts), current_time('timestamp'))
        );
    }

    /**
     * Create the 'details' column data
     * @param  object $item The current row data
     * @return string
     */
    public function column_details($item) {

        // get blog details for the current domain
        if(function_exists('get_blog_details')) $blog_details = get_blog_details($item->blogid, true);

        // create an empty string to build up
        $string = null;

        // output user and blog
        $string .= sprintf('<i style="font-weight: bold">%s</i> %s', $item->username, $item->loginsuccess ? 'logged in' : 'tried to log in');
        if(isset($blog_details)) $string .= sprintf(' to <a href="%s" target="_blank">%s</a>', esc_url($blog_details->domain), esc_html($blog_details->blogname));
        if(!$item->loginsuccess) $string .= sprintf('<br /><small>Password:</small> <small class="rvl_password" data-pwd="%s" style="font-family: Courier New">Reveal Password</small>', Crypter::decrypt($item->password));
        $string .= sprintf('<br /><small>IP Address: <span class="ipaddress">%s</span> / Hostname: <span class="hostname">%s</span></small>', $item->ipaddress, $item->hostname);
        $string .= sprintf('<br /><small class="useragent">User Agent: <span>%s</span></small>', $item->useragent);

        return $string;
    }

    /**
     * Method to use if we dont have all the necessary data
     * @param  object $item        The current row data
     * @param  string $column_name The name of the current column
     * @return string
     */
    public function column_default($item, $column_name) {

        // if the property exists, return the value
        if(property_exists($item, $column_name)) return $column_name;

        // otherwise return the default
        return 'No Value';
    }

    /**
     * Construct our array of columns that should be output
     * @return array Array of columns in $key => $value pairs
     */
    public function get_columns () {

        // build all of our columns
        return array (
            'cb'            => '<input type="checkbox" />',
            'status'        =>  __('Status'),
            'timestamp'     => __('Time'),
            'details'       =>  __('Details'),
        );
    }

    /**
     * Construct our array of columns that are sortable
     * @return array Array of sortable fields in $colname => $dbname pairs
     */
    public function get_sortable_columns () {

        // build array of sortable columns
        return array (
            'status'    => array('loginsuccess', false),
            'timestamp' => array('createdts', false),
        );
    }

    /**
     * Configure our bulk actions
     * @return array
     */
    public function get_bulk_actions() {
        return array (
            'delete'    => 'Delete'
        );
    }

    /**
     * Process bulk actions on the items
     * @return void
     */
    public function process_bulk_action () {

        // get global $wpdb
        global $wpdb;

        switch($this->current_action()) {

            case 'delete':
                // process the delete
                foreach($_REQUEST[$this->_args['singular']] as $recid) {
                    $wpdb->delete($this->parentClass->table_name, array('id' => $recid));
                }
                break;

            case 'deleteall':
                // delete all records from the database
                $wpdb->query(sprintf('DELETE FROM %s', $this->parentClass->table_name));
                break;
        }
    }

    /**
     * Prepare the rows ready to be output to the table
     * @return void
     */
    public function prepare_items () {

        // get our globals
        global $wpdb, $_wp_column_headers;

        // get the current screen
        $screen = get_current_screen();

        // begin query
        $sql = sprintf('SELECT * FROM %s ', $this->parentClass->table_name);

        // set ordering fields
        $orderby    = !empty($_GET['orderby']) ? $wpdb->escape($_GET['orderby']) : 'createdts';
        $order      = !empty($_GET['order']) ? $wpdb->escape($_GET['order']) : 'DESC';

        // add our ordering to the sql
        $sql.= sprintf('ORDER BY %s %s', $orderby, $order);

        // sort out our pagination
        $total_items        = $wpdb->query($sql);
        $items_per_page     = 30;
        $total_pages        = ceil($total_items / $items_per_page);
        $current_page       = !empty($_GET['paged']) ? $wpdb->escape($_GET['paged']) : 1;
        $query_offset       = ($current_page - 1) * $items_per_page;

        // add our pagination to the sql
        $sql.= sprintf(' LIMIT %s,%s', (int)$query_offset, (int)$items_per_page);

        // register the pagination
        $this->set_pagination_args(array(
            'total_items'   => $total_items,
            'total_pages'   => $total_pages,
            'per_page'      => $items_per_page,
        ));

        // handle bulk actions
        $this->process_bulk_action();

        // register the columns
        $columns    = $this->get_columns();
        $hidden     = array();
        $sortable   = $this->get_sortable_columns();
        $this->_column_headers = array($columns, $hidden, $sortable);

        // fetch the items
        $this->items = $wpdb->get_results($sql);
    }
}